/* REXX */

/*************************************************************************

   eqsl_fldigi.rex

   Version 1.0.5
   
   9 October 2011

   10 October 2011 added comments about how to use this script.

   17 October 2011 Edited band determination to abort with a message
                   if the frequency does not fit any in the list.

   19 October 2011 Changed valid frequencies to match those in the adif 
                   specifications.
                   
   9 November 2011 Added resend and message features suggested and
                   programmed by Ben Parkes, G3XDH.  Also added more
                   comments on how to use this script.

   copyright David Rife, AD2Q, 2011
   copyright David Freese, W1HKJ, 2011

   This REXX script, which runs on Linux or Windows and requires a REXX 
   interpreter, enables fldigi to call the default web browser to upload 
   QSO information to www.eqsl.cc.
   
   This script has been tested on Windows using the "Regina" interpreter.

   This script has been tested on Linux using the "Regina" rexx distri-
   bution and also ooRexx, an object oriented Rexx.

   The functions of this script are unique to fldigi.

   A separate data file contains the user's eqsl.cc ID and password.
   If not found, the file will be created based on user input and the
   script will abort.

   This file may be put in any directory where fldigi can access the
   file.

   The default location for the two files used or generated by this
   script is defined by the variable 'home'.  The 'home' location may
   be edited to suit the user.

   It is recommended that you run this script from a command line the
   first time it used.  That will ease the creation of the data file.
   You may create the data file separately using a text editor.  The
   file name is eqsldata.txt and is located in

     C:\regina on Windows
     $HOME on Linux, BSD or OS X

   The file contains two lines
     line 1: eQSL login callsign
     line 2: eQSL password

   The Windows command in a fldigi macro is:
<EXEC>regina c:\regina\eqsl_fldigi.rex <FREQ> <CALL> <MODE> <IZDT> <RST> [message]</EXEC>

   The Linux macro command is:
<EXEC>rexx <pathtofile>/eqsl_fldigi.rex <FREQ> <CALL> <MODE> <IZDT> <RST> [message]</EXEC>

   The optional message field replaces the default message with the one on the command line:
<EXEC>regina c:\regina\eqsl_fldigi.rex <FREQ> <CALL> <MODE> <IZDT> <RST> Please qsl via eqsl.cc </EXEC>
   
   This Windows command line lets one read the outfile, which is defined below.
<EXEC>notepad c:\regina\eqslfile.txt </EXEC>
   
   If the call field is "TEST" or the script aborts, then the output is sent 
   to the outfile.

   If no RST is defined in fldigi when this script is invoked, the
   RST_SENT field, which is optional, will not be sent to eQSL.cc.

   If no argument is given, then the previous QSO information is sent:
<EXEC>regina c:\regina\eqsl_fldigi.rex</EXEC>   

   This script must be run while the QSO information is still showing
   at the top of the fldigi screen; otherwise the variables used in the
   macro command do not have values.  Either set fldigi to not clear the
   QSO information after logging or use this script before logging a QSO.
   
   This script requires fldigi version 3.21.17 or later.  Earlier versions
   do not send the command line values with the EXEC function.

   Some mode and baud combinations are not valid for adif 2.2.7,
   which is the current specification dated 2010-11-3, and are
   not accepted by eQSL.cc.  This script will not send invalid QSL
   information.

   This script is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This script is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details

   <http://www.gnu.org/licenses/>.

*************************************************************************/

opsys = "Windows"
home = GETENV('HOME')
IF POS("/", home) > 0 THEN DO
    opsys = "Linux"
    home = home||"/"
END
ELSE
    home = "C:\regina\"

/*  define the data and output files */

datafile = home"eqsldata.txt"
outfile  = home"eqslfile.txt"
kfile = home"kstring.txt"

CALL xdelete outfile

/*  Check for existence of the data file and create one IF needed. */

IF STREAM(datafile, "C", "QUERY EXISTS")\="" THEN DO
    userid = linein(datafile)
    userpw = linein(datafile)
    CALL xclose datafile
    END
ELSE DO
    SAY "Enter your eqsl.cc user ID: "
    userid = linein()
    SAY "Enter your eqsl.cc user password: "
    userpw = linein()
    CALL lineout datafile, userid
    CALL lineout datafile, userpw
    CALL xclose datafile
    CALL xwrite "Initialized the data file."
    EXIT
    END

/* parse input parameters, convert to upper case */

parse arg xfreq xcall xmode xdate xtime xrsq xmsg

CALL xwrite 'fldigi command line: 'xfreq' 'xcall' 'xmode' 'xdate' 'xtime' 'xrsq' 'xmsg

if xmsg = "" then xmsg = "Please QSL via eQSL .. Thank you .. 73" /* Default message */

/* if no arguments this is a re-send request */
 
if xfreq = "" then do        /* if no arguments, send the last QSO to eQSL */
    kstring = LINEIN(kfile)  /* get last QSO details */
    CALL xclose kfile  
    IF kstring = "" then do 
      CALL xwrite 'No kstring found.'
      EXIT
      END
    ADDRESS SYSTEM kstring   /* send to eQSL via default browser */
    EXIT
    END

call xdelete kfile           /* delete last QSO details */

/* abort if command line is invalid */

IF POS(":",xtime) = 0 THEN DO
    CALL xwrite 'Invalid time string: ['xtime']'
    EXIT
    END
    
xcall = TRANSLATE(xcall)
xmode = TRANSLATE(xmode)
xdate = TRANSLATE(xdate)

/*  find the band */
/* handles MHz, Khz and Hz for xfreq parameter */

IF xfreq > 1000 THEN xfreq = xfreq / 1000
IF xfreq > 1000 THEN xfreq = xfreq / 1000
IF xfreq > 1000 THEN xfreq = xfreq / 1000

    xband = "INVALID" 
 
    IF xfreq >= 1.8    & xfreq < 2.0    THEN xband = "160m"
    IF xfreq >= 3.5    & xfreq < 4.0    THEN xband = "80m"
    IF xfreq >= 5.102  & xfreq <= 5.405 THEN xband = "60m"
    IF xfreq >= 7.0    & xfreq < 7.3    THEN xband = "40m"
    IF xfreq >= 10.0   & xfreq < 10.15  THEN xband = "30m"
    IF xfreq >= 14.0   & xfreq < 14.35  THEN xband = "20m"
    IF xfreq >= 18.068 & xfreq < 18.168 THEN xband = "17m"
    IF xfreq >= 21.0   & xfreq < 21.45  THEN xband = "15m"
    IF xfreq >= 24.89  & xfreq < 24.99  THEN xband = "12m"
    IF xfreq >= 28.0   & xfreq < 29.7   THEN xband = "10m"
    IF xfreq >= 50.0   & xfreq < 54.0   THEN xband = "6m"
    IF xfreq >= 144.0  & xfreq < 148.0  THEN xband = "2m"
    IF xfreq >= 222.0  & xfreq < 225.0  THEN xband = "1.25m"
    IF xfreq >= 420.0  & xfreq < 450.0  THEN xband = "70cm"

    IF xband = "INVALID" THEN DO
        CALL xwrite "Invalid frequency."
        EXIT
        END
        
/*  remove hyphens in xdate string */

xdate = SUBSTR(xdate, 1, 4)||SUBSTR(xdate, 6, 2)||SUBSTR(xdate, 9, 2)

/*  remove colon and Z from xtime string */

xtime = SUBSTR(xtime, 1, 2)||SUBSTR(xtime, 4, 2)

/*  convert mode to correct format for adIF */
/*  some modes do not need conversion       */

IF POS("BPSK", xmode) > 0  THEN xmode = SUBSTR(xmode, 2)
IF POS("THRBX", xmode) > 0  THEN xmode = "THRBX"
IF POS("THROB", xmode) > 0  THEN xmode = "THRB"
IF POS("MT63", xmode) > 0  THEN xmode = "MT63"
IF POS("DOMX", xmode) > 0 THEN xmode = "DOMINO"
IF POS("CTSTIA", xmode) > 0 THEN xmode = "CONTESTI"
IF POS("MFSK4", xmode) > 0 |,
   POS("MFSK11", xmode) > 0 |,
   POS("MFSK22", xmode) > 0 |,
   POS("MFSK31", xmode) > 0 |,
   POS("MFSK32", xmode) > 0 |,
   POS("MFSK64", xmode) > 0 THEN xmode = "MFSK16"
ymode = xmode
IF POS("HELL", ymode) > 0 THEN xmode = "HELL"
IF POS("HELL80", ymode) > 0  THEN xmode = "HELL80"

/* Abort if xmode is not valid for adif */

xv1 = 'AM AMTORFEC ASCI ATV CHIP64 CHIP128 CLO CONTESTI CW DSTAR DOMINO DOMINOF FAX'
xv2 = 'FM FMHELL FSK31 FSK441 GTOR HELL HELL80 HFSK JT44 JT4A JT4B JT4C JT4D JT4E'
xv3 = 'JT4F JT4G JT65 JT65A JT65B JT65C JT6M MFSK8 MFSK16'
xv4 = 'MT63 OLIVIA PAC PAC2 PAC3 PAX PAX2 PCW PKT PSK10 PSK31 PSK63 PSK63F PSK125'
xv5 = 'PSKAM10 PSKAM31 PSKAM50 PSKFEC31 PSKHELL Q15 QPSK31 QPSK63 QPSK125'
xv6 = 'ROS RTTY RTTYM SSB SSTV THRB THOR THRBX TOR VOI WINMOR WSPR'
xvalid = xv1' 'xv2' 'xv3' 'xv4' 'xv5' 'xv6
IF POS(xmode, xvalid) = 0 THEN DO
    CALL xwrite 'Invalid mode: 'xmode
    EXIT
    END

/* generate the string to call default web browser */

s1 = "http://www.eqsl.cc/qslcard/importADIF.cfm?ADIFdata=upload%20%3cadIF_ver:5%3e2.1.9"
s2 = "%3cEQSL_USER:"||LENGTH(userid)||"%3e"||userid||"%3cEQSL_PSWD:"||LENGTH(userpw)||"%3e"||userpw||"%3cPROGRAMID:6%3eFLDIGI%3cEOH%3e"
s3 = "%3cBAND:"||LENGTH(xband)||"%3e"||xband
s4 = "%3cCALL:"||LENGTH(xcall)||"%3e"||xcall
s5 = "%3cMODE:"||LENGTH(xmode)||"%3e"||xmode
s6 = "%3cQSO_DATE:"||LENGTH(xdate)||"%3e"||xdate
s7 = "%3cTIME_ON:"||LENGTH(xtime)||"%3e"||xtime
s8 = "%3cRST_SENT:"||LENGTH(xrsq)||"%3e"||xrsq
s9 = "%3cQSLMSG:"||LENGTH(xmsg)||"%3e"||xmsg 

IF datatype(xrsq) <>  'NUM' THEN do
  xmsg = xrsq || " " || xmsg
  s9 = "%3cQSLMSG:"||LENGTH(xmsg)||"%3e"||xmsg 
  gstring = s1||s2||s3||s4||s5||s6||s7||s9 || "%3cEOR%3e"
  end
ELSE
  gstring = s1||s2||s3||s4||s5||s6||s7||s8||s9 || "%3cEOR%3e"

IF opsys = "Linux" THEN
kstring = "xdg-open "||gstring
ELSE
kstring = "rundll32.exe url.dll,FileProtocolHandler "||gstring

/* Save kstring for resend request. */

CALL LINEOUT kfile, kstring
CALL xclose kfile

/* Send the string to a file if call is "TEST", otherwise call web browser. */

IF xcall = "TEST" THEN 
    CALL xwrite kstring
ELSE
    ADDRESS SYSTEM kstring
EXIT

/* helpful procedures */

xclose: PROCEDURE
    CALL STREAM arg(1), "C", "CLOSE"
    RETURN

xdelete: PROCEDURE expose opsys
    dfile = arg(1)
    IF STREAM(dfile, "C", "QUERY EXISTS")\="" THEN DO
      IF opsys = "Linux" THEN
        ADDRESS SYSTEM "rm "dfile
      ELSE
        ADDRESS SYSTEM "del "dfile
    END
    RETURN

xwrite: PROCEDURE EXPOSE outfile
    CALL LINEOUT outfile, arg(1)
    CALL xclose outfile
    RETURN
